@@ -2,11 +2,9 @@
 // requesting certificates with ACME, typically from Let's Encrypt.
 package autotls
 
-// We only do tls-alpn-01. For http-01, we would have to start another
-// listener. For DNS we would need a third party tool with an API that can make
-// the DNS changes, as we don't want to link in dozens of bespoke API's for DNS
-// record manipulation into mox. We can do http-01 relatively easily. It could
-// be useful to not depend on a single mechanism.
+// We do tls-alpn-01, and also http-01. For DNS we would need a third party tool
+// with an API that can make the DNS changes, as we don't want to link in dozens of
+// bespoke API's for DNS record manipulation into mox.
 
 import (
 	"bytes"
@@ -257,8 +255,8 @@ var errHostNotAllowed = errors.New("autotls: host not in allowlist")
 
 // HostPolicy decides if a host is allowed for use with ACME, i.e. whether a
 // certificate will be returned if present and/or will be requested if not yet
-// present. Only hosts added with AllowHostname are allowed. During shutdown, no
-// new connections are allowed.
+// present. Only hosts added with SetAllowedHostnames are allowed. During shutdown,
+// no new connections are allowed.
 func (m *Manager) HostPolicy(ctx context.Context, host string) (rerr error) {
 	log := xlog.WithContext(ctx)
 	defer func() {
@@ -272,6 +270,12 @@ func (m *Manager) HostPolicy(ctx context.Context, host string) (rerr error) {
 	default:
 	}
 
+	xhost, _, err := net.SplitHostPort(host)
+	if err == nil {
+		// For http-01, host may include a port number.
+		host = xhost
+	}
+
 	d, err := dns.ParseDomain(host)
 	if err != nil {
 		return fmt.Errorf("invalid host: %v", err)
