@@ -16,7 +16,7 @@ low-maintenance self-hosted email.
 	mox serve
 	mox quickstart [-existing-webserver] [-hostname host] user@domain [user | uid]
 	mox stop
-	mox setaccountpassword address
+	mox setaccountpassword account
 	mox setadminpassword
 	mox loglevels [level [pkg]]
 	mox queue list
@@ -57,6 +57,7 @@ low-maintenance self-hosted email.
 	mox dmarc lookup domain
 	mox dmarc parsereportmsg message ...
 	mox dmarc verify remoteip mailfromaddress helodomain < message
+	mox dmarc checkreportaddrs domain
 	mox dnsbl check zone ip
 	mox dnsbl checkhealth zone
 	mox mtasts lookup domain
@@ -76,6 +77,7 @@ low-maintenance self-hosted email.
 	mox ensureparsed account
 	mox recalculatemailboxcounts account
 	mox message parse message.eml
+	mox reassignthreads [account]
 
 Many commands talk to a running mox instance, through the ctl file in the data
 directory. Specify the configuration file (that holds the path to the data
@@ -154,9 +156,11 @@ password itself, are stored in the account database. The stored secrets are for
 authentication with: scram-sha-256, scram-sha-1, cram-md5, plain text (bcrypt
 hash).
 
-Any email address configured for the account can be used.
+The parameter is an account name, as configured under Accounts in domains.conf
+and as present in the data/accounts/ directory, not a configured email address
+for an account.
 
-	usage: mox setaccountpassword address
+	usage: mox setaccountpassword account
 
 # mox setadminpassword
 
@@ -360,7 +364,7 @@ back to should an upgrade fail. Simply copying files in the data directory
 while mox is running can result in unusable database files.
 
 Message files never change (they are read-only, though can be removed) and are
-hardlinked so they don't consume additional space. If hardlinking fails, for
+hard-linked so they don't consume additional space. If hardlinking fails, for
 example when the backup destination directory is on a different file system, a
 regular copy is made. Using a destination directory like "data/tmp/backup"
 increases the odds hardlinking succeeds: the default systemd service file
@@ -665,6 +669,18 @@ can be found in message headers.
 
 	usage: mox dmarc verify remoteip mailfromaddress helodomain < message
 
+# mox dmarc checkreportaddrs
+
+For each reporting address in the domain's DMARC record, check if it has opted into receiving reports (if needed).
+
+A DMARC record can request reports about DMARC evaluations to be sent to an
+email/http address. If the organizational domains of that of the DMARC record
+and that of the report destination address do not match, the destination
+address must opt-in to receiving DMARC reports by creating a DMARC record at
+<dmarcdomain>._report._dmarc.<reportdestdomain>.
+
+	usage: mox dmarc checkreportaddrs domain
+
 # mox dnsbl check
 
 Test if IP is in the DNS blocklist of the zone, e.g. bl.spamcop.net.
@@ -793,7 +809,7 @@ Change the IMAP UID validity of the mailbox, causing IMAP clients to refetch mes
 This can be useful after manually repairing metadata about the account/mailbox.
 
 Opens account database file directly. Ensure mox does not have the account
-+open, or is not running.
+open, or is not running.
 
 	usage: mox bumpuidvalidity account [mailbox]
 
@@ -869,6 +885,40 @@ incorrect. This command will find, fix and print them.
 Parse message, print JSON representation.
 
 	usage: mox message parse message.eml
+
+# mox reassignthreads
+
+Reassign message threads.
+
+For all accounts, or optionally only the specified account.
+
+Threading for all messages in an account is first reset, and new base subject
+and normalized message-id saved with the message. Then all messages are
+evaluated and matched against their parents/ancestors.
+
+Messages are matched based on the References header, with a fall-back to an
+In-Reply-To header, and if neither is present/valid, based only on base
+subject.
+
+A References header typically points to multiple previous messages in a
+hierarchy. From oldest ancestor to most recent parent. An In-Reply-To header
+would have only a message-id of the parent message.
+
+A message is only linked to a parent/ancestor if their base subject is the
+same. This ensures unrelated replies, with a new subject, are placed in their
+own thread.
+
+The base subject is lower cased, has whitespace collapsed to a single
+space, and some components removed: leading "Re:", "Fwd:", "Fw:", or bracketed
+tag (that mailing lists often add, e.g. "[listname]"), trailing "(fwd)", or
+enclosing "[fwd: ...]".
+
+Messages are linked to all their ancestors. If an intermediate parent/ancestor
+message is deleted in the future, the message can still be linked to the earlier
+ancestors. If the direct parent already wasn't available while matching, this is
+stored as the message having a "missing link" to its stored ancestors.
+
+	usage: mox reassignthreads [account]
 */
 package main
 
