@@ -1,6 +1,7 @@
 package http
 
 import (
+	"net"
 	"net/http"
 	"strings"
 	"time"
@@ -12,15 +13,24 @@ import (
 )
 
 func mtastsPolicyHandle(w http.ResponseWriter, r *http.Request) {
-	log := xlog.WithCid(mox.Cid())
+	log := func() *mlog.Log {
+		return xlog.WithContext(r.Context())
+	}
 
-	if !strings.HasPrefix(r.Host, "mta-sts.") {
+	host := strings.ToLower(r.Host)
+	if !strings.HasPrefix(host, "mta-sts.") {
 		http.NotFound(w, r)
 		return
 	}
-	domain, err := dns.ParseDomain(strings.TrimPrefix(r.Host, "mta-sts."))
+	host = strings.TrimPrefix(host, "mta-sts.")
+	nhost, _, err := net.SplitHostPort(host)
+	if err == nil {
+		// Only relevant for when host has a port.
+		host = nhost
+	}
+	domain, err := dns.ParseDomain(host)
 	if err != nil {
-		log.Errorx("mtasts policy request: bad domain", err, mlog.Field("host", r.Host))
+		log().Errorx("mtasts policy request: bad domain", err, mlog.Field("host", host))
 		http.NotFound(w, r)
 		return
 	}
@@ -41,7 +51,7 @@ func mtastsPolicyHandle(w http.ResponseWriter, r *http.Request) {
 		}
 		d, err := dns.ParseDomain(s)
 		if err != nil {
-			log.Errorx("bad domain in mtasts config", err, mlog.Field("domain", s))
+			log().Errorx("bad domain in mtasts config", err, mlog.Field("domain", s))
 			http.Error(w, "500 - internal server error - invalid domain in configuration", http.StatusInternalServerError)
 			return
 		}
