@@ -595,7 +595,7 @@ func serve(listenerName string, cid int64, hostname dns.Domain, tlsConfig *tls.C
 		} else {
 			c.log.Error("unhandled panic", mlog.Field("err", x))
 			debug.PrintStack()
-			metrics.PanicInc("smtpserver")
+			metrics.PanicInc(metrics.Smtpserver)
 		}
 	}()
 
@@ -773,15 +773,19 @@ func (c *conn) cmdEhlo(p *parser) {
 
 // ../rfc/5321:1783
 func (c *conn) cmdHello(p *parser, ehlo bool) {
-	// ../rfc/5321:1827, though a few paragraphs earlier at ../rfc/5321:1802 is a claim
-	// additional data can occur.
-	p.xspace()
 	var remote dns.IPDomain
-	if ehlo {
-		remote = p.xipdomain(true)
+	if c.submission && !moxvar.Pedantic {
+		// Mail clients regularly put bogus information in the hostname/ip. For submission,
+		// the value is of no use, so there is not much point in annoying the user with
+		// errors they cannot fix themselves. Except when in pedantic mode.
+		remote = dns.IPDomain{IP: c.remoteIP}
 	} else {
-		remote = dns.IPDomain{Domain: p.xdomain()}
-		if !c.submission {
+		p.xspace()
+		if ehlo {
+			remote = p.xipdomain(true)
+		} else {
+			remote = dns.IPDomain{Domain: p.xdomain()}
+
 			// Verify a remote domain name has an A or AAAA record, CNAME not allowed. ../rfc/5321:722
 			cidctx := context.WithValue(mox.Context, mlog.CidKey, c.cid)
 			ctx, cancel := context.WithTimeout(cidctx, time.Minute)
@@ -792,9 +796,15 @@ func (c *conn) cmdHello(p *parser, ehlo bool) {
 			}
 			// For success or temporary resolve errors, we'll just continue.
 		}
+		// ../rfc/5321:1827
+		// Though a few paragraphs earlier is a claim additional data can occur for address
+		// literals (IP addresses), although the ABNF in that document does not allow it.
+		// We allow additional text, but only if space-separated.
+		if len(remote.IP) > 0 && p.space() {
+			p.remainder() // ../rfc/5321:1802 ../rfc/2821:1632
+		}
+		p.xend()
 	}
-	p.remainder() // ../rfc/5321:1802
-	p.xend()
 
 	// Reset state as if RSET command has been issued. ../rfc/5321:2093 ../rfc/5321:2453
 	c.rset()
@@ -1893,6 +1903,7 @@ func (c *conn) deliver(ctx context.Context, recvHdrFor func(string) string, msgW
 			if x != nil {
 				c.log.Error("dkim verify panic", mlog.Field("err", x))
 				debug.PrintStack()
+				metrics.PanicInc(metrics.Dkimverify)
 			}
 		}()
 		defer wg.Done()
@@ -1926,8 +1937,9 @@ func (c *conn) deliver(ctx context.Context, recvHdrFor func(string) string, msgW
 		defer func() {
 			x := recover() // Should not happen, but don't take program down if it does.
 			if x != nil {
-				c.log.Error("dkim verify panic", mlog.Field("err", x))
+				c.log.Error("spf verify panic", mlog.Field("err", x))
 				debug.PrintStack()
+				metrics.PanicInc(metrics.Spfverify)
 			}
 		}()
 		defer wg.Done()
@@ -2114,16 +2126,16 @@ func (c *conn) deliver(ctx context.Context, recvHdrFor func(string) string, msgW
 	// Prepare for analyzing content, calculating reputation.
 	ipmasked1, ipmasked2, ipmasked3 := ipmasked(c.remoteIP)
 	var verifiedDKIMDomains []string
+	dkimSeen := map[string]bool{}
 	for _, r := range dkimResults {
 		// A message can have multiple signatures for the same identity. For example when
 		// signing the message multiple times with different algorithms (rsa and ed25519).
-		seen := map[string]bool{}
 		if r.Status != dkim.StatusPass {
 			continue
 		}
 		d := r.Sig.Domain.Name()
-		if !seen[d] {
-			seen[d] = true
+		if !dkimSeen[d] {
+			dkimSeen[d] = true
 			verifiedDKIMDomains = append(verifiedDKIMDomains, d)
 		}
 	}
@@ -2315,7 +2327,7 @@ func (c *conn) deliver(ctx context.Context, recvHdrFor func(string) string, msgW
 		if !a.accept {
 			conf, _ := acc.Conf()
 			if conf.RejectsMailbox != "" {
-				present, messageid, messagehash, err := rejectPresent(log, acc, conf.RejectsMailbox, m, dataFile)
+				present, _, messagehash, err := rejectPresent(log, acc, conf.RejectsMailbox, m, dataFile)
 				if err != nil {
 					log.Errorx("checking whether reject is already present", err)
 				} else if !present {
@@ -2324,7 +2336,6 @@ func (c *conn) deliver(ctx context.Context, recvHdrFor func(string) string, msgW
 					// Regular automatic junk flags configuration applies to these messages. The
 					// default is to treat these as neutral, so they won't cause outright rejections
 					// due to reputation for later delivery attempts.
-					m.MessageID = messageid
 					m.MessageHash = messagehash
 					acc.WithWLock(func() {
 						hasSpace := true
@@ -2378,7 +2389,7 @@ func (c *conn) deliver(ctx context.Context, recvHdrFor func(string) string, msgW
 			}
 		}
 
-		// If a forwarded message and this is a first-time sender, wait before actually
+		// If this is a first-time sender and not a forwarded message, wait before actually
 		// delivering. If this turns out to be a spammer, we've kept one of their
 		// connections busy.
 		if delayFirstTime && !m.IsForward && a.reason == reasonNoBadSignals && c.firstTimeSenderDelay > 0 {
@@ -2420,8 +2431,8 @@ func (c *conn) deliver(ctx context.Context, recvHdrFor func(string) string, msgW
 			log.Info("incoming message delivered", mlog.Field("reason", a.reason), mlog.Field("msgfrom", msgFrom))
 
 			conf, _ := acc.Conf()
-			if conf.RejectsMailbox != "" && messageID != "" {
-				if err := acc.RejectsRemove(log, conf.RejectsMailbox, messageID); err != nil {
+			if conf.RejectsMailbox != "" && m.MessageID != "" {
+				if err := acc.RejectsRemove(log, conf.RejectsMailbox, m.MessageID); err != nil {
 					log.Errorx("removing message from rejects mailbox", err, mlog.Field("messageid", messageID))
 				}
 			}
